@@ -20,6 +20,7 @@ from __future__ import division, print_function, absolute_import
 import enum
 import math
 import random
+import string
 import datetime as dt
 import operator
 from uuid import UUID
@@ -108,7 +109,7 @@ __all__ = [
     'recursive', 'composite',
     'shared', 'runner', 'data',
     'deferred',
-    'from_type', 'register_type_strategy',
+    'from_type', 'register_type_strategy', 'emails',
 ]
 
 _strategies = set()
@@ -525,7 +526,7 @@ def sampled_from(elements):
     1 values with 10, and sampled_from((1, 10)) will shrink by trying to
     replace 10 values with 1.
     """
-    values = check_sample(elements)
+    values = check_sample(elements, 'sampled_from')
     if not values:
         return nothing()
     if len(values) == 1:
@@ -1486,13 +1487,13 @@ class PermutationStrategy(SearchStrategy):
 @defines_strategy
 def permutations(values):
     # type: (Sequence[T]) -> SearchStrategy[List[T]]
-    """Return a strategy which returns permutations of the collection
+    """Return a strategy which returns permutations of the ordered collection
     ``values``.
 
     Examples from this strategy shrink by trying to become closer to the
     original order of values.
     """
-    values = list(values)
+    values = check_sample(values, 'permutations')
     if not values:
         return builds(list)
 
@@ -1533,6 +1534,10 @@ def datetimes(
     package, but provides all timezones in the Olsen database.  If you want to
     allow naive datetimes, combine strategies like ``none() | timezones()``.
 
+    :py:func:`hypothesis.extra.dateutil.timezones` requires the
+    :pypi:`python-dateutil` package, and similarly provides all timezones
+    there.
+
     Alternatively, you can create a list of the timezones you wish to allow
     (e.g. from the standard library, ``datetutil``, or ``pytz``) and use
     :py:func:`sampled_from`.  Ensure that simple values such as None or UTC
@@ -1821,7 +1826,7 @@ class Chooser(object):
     def __call__(self, values):
         if not values:
             raise IndexError('Cannot choose from empty sequence')
-        result = choice(self.data, check_sample(values))
+        result = choice(self.data, check_sample(values, 'choices'))
         with self.build_context.local():
             self.choice_count += 1
             note('Choice #%d: %r' % (self.choice_count, result))
@@ -2058,4 +2063,22 @@ def deferred(definition):
     return DeferredStrategy(definition)
 
 
+@defines_strategy_with_reusable_values
+def emails():
+    """A strategy for generating email addresses as unicode strings. The
+    address format is specific in :rfc:`5322#section-3.4.1`. Values shrink
+    towards shorter local-parts and host domains.
+
+    This strategy is useful for generating "user data" for tests, as
+    mishandling of email addresses is a common source of bugs. Future
+    updates will generate more complicated addresses allowed by the RFC.
+    """
+    from hypothesis.provisional import domains
+    local_chars = string.ascii_letters + string.digits + "!#$%&'*+-/=^_`{|}~"
+    local_part = text(local_chars, min_size=1, max_size=64)
+    # TODO: include dot-atoms, quoted strings, escaped chars, etc in local part
+    return builds(u'{}@{}'.format, local_part, domains()).filter(
+        lambda addr: len(addr) <= 255)
+
+
 assert _strategies.issubset(set(__all__)), _strategies - set(__all__)
